# ARST打卡第188周[188/521]

<!--more-->

## Algorithm
lc1780_判断一个数字是否可以表示成三的幂的和

思路: 
直接先判断3的多少幂能超过数据范围`[1, 1e7]`，然后直接倒着减

```cpp
// 2022年12月09日19:53:55 感觉这个题目就是打表，因为n在[1, 1e7]
// 先打表看看3的多少次方大于1e7: src: 14348907, i: 15.
// 2022年12月09日20:05:55 AC，12分钟，有点慢了
bool checkPowersOfThree(int n) {
    int src = 1;
    int nums[20];
    nums[0] = src;
    for (int i = 1;; i++) {
        src *= 3;
        if (src > 1e7) {
            break;
        }
        nums[i] = src;
    }
    for (int i = 14; i >= 0; i--) {
        if (n >= nums[i]) {
            n -= nums[i];
        }
    }
    return n == 0;
}
```

当然看了参考答案的思路也很简单

思路: 转化为3进制，每个位上都是3的多少次方，只能用1或者0作为这个次方的系数，不能为2即可

```cpp
bool checkPowersOfThree(int n) {
    while (n) {
        if (n % 3 == 2) {
            return false;
        }
        n /= 3;
    }
    return true;
}
```

## Review
TED演讲：远离智能手机,更少的资源让你有更多的时间进行创作

<div align = "center">
    <iframe  width="100%" height="700" src="//player.bilibili.com/player.html?aid=306121210&bvid=BV1GP411T7V6&cid=916072077&page=1" frameborder="0" allow="autoplay; encrypted-media" allowfullscreen></iframe>
</div>

如果你拥有更少的有趣的物品，那么你将会拥有更多的无聊的时间，那么你将拥有更低的幸福阈值

这样你很容易对一些看起来很枯燥的事情感到很有兴趣，从而让你更有可能做更多更难的事情

因此更加容易进行创作，更加容易创造一些价值

所以有趣的事物可能让我们终日沉迷，而远离干扰才能让我们创造更多的价值，从而生活得更加从容

因此建议以后多多远离干扰，当然这需要自律，但是自律的生活才是自己对自身的真正自由

## Tips
[Redis Scan原理解析与踩坑](https://www.lixueduan.com/posts/redis/redis-scan/)

## Share-小心weak_ptr的lock()生成的临时shared_ptr导致reset释放不掉内存
### 个人通过chatGPT学习weak_ptr，得到如下答案
**weak_ptr用法**

weak_ptr 是一种智能指针类型，它指向一个 shared_ptr 指向的内存，但不会增加共享指针的引用计数。

因为 weak_ptr 不会增加共享指针的引用计数，所以当共享指针释放内存时，weak_ptr 也会失效，无法通过 weak_ptr 访问内存。

使用 weak_ptr 的方法如下：

引入 weak_ptr 头文件：#include <memory>

创建 weak_ptr 实例：std::weak_ptr<int> wptr = ptr;

访问 weak_ptr 指向的内存：std::shared_ptr<int> sptr = wptr.lock();

如果 weak_ptr 无效，则 sptr 的值为 nullptr。

例如：

```cpp
#include <iostream>
#include <memory>

int main()
{
    // 创建一个指向 int 类型的共享指针
    std::shared_ptr<int> ptr = std::make_shared<int>(10);

    // 创建一个 weak_ptr
    std::weak_ptr<int> wptr = ptr;

    // 访问 weak_ptr 指向的内存
    std::shared_ptr<int> sptr = wptr.lock();
    if (sptr)
    {
        std::cout << *sptr << std::endl;
    }

    // 释放共享指针指向的内存
    ptr.reset();

    // 再次访问 weak_ptr 指向的内存
    sptr = wptr.lock();
    if (!sptr)
    {
        std::cout << "weak_ptr is invalid" << std::endl;
    }

    return 0;
}
```

### 运行得不到 `weak_ptr is invalid`的结果

经过自己打印日志发现`
lock()`生成的临时`shared_ptr`可能会导致`reset()`内存释放不掉

**具体原因是: `shared_ptr<int> sptr = wptr.lock();`会新建一个新的对应内存的引用，所以导致内存块不释放**
    
```cpp
#include <iostream>
#include <unistd.h>
#include <memory>
using namespace std;

int main() {
    // 创建一个指向 int 类型的共享指针
    shared_ptr<int> ptr = make_shared<int>(10);

    // 创建一个 weak_ptr
    weak_ptr<int> wptr = ptr;

    // 访问 weak_ptr 指向的内存
    // 被chatGPT的示例代码坑惨了，这里会新建一个新的对应内存的引用，所以导致内存块不释放
    shared_ptr<int> sptr = wptr.lock();
    if (sptr) {
        cout << *sptr << endl;
    }

     // 输出共享指针的引用计数
    cout << "before reset: " << ptr.use_count() << endl;

    // 释放共享指针指向的内存
    ptr.reset();

    // 输出共享指针的引用计数
    cout << "after reset: " << ptr.use_count() << endl;
    cout << "after reset: sptr count " << sptr.use_count() << endl;
    sptr.reset();
    cout << "after sptr reset: sptr count " << sptr.use_count() << endl;

    cout << "expired ? : " <<  wptr.expired() << endl;
    // 再次访问 weak_ptr 指向的内存
    sptr = wptr.lock();
    cout << "after reset: " << ptr.use_count() << endl;
    if (!sptr) {
        cout << "weak_ptr is invalid" << endl;
    } else {
        cout << sptr << " " << *sptr << endl;
    }
    while(!wptr.expired()) {
        sleep(1);
        cout << "not expired, sleep 1s..." << endl;
    }
    if (!sptr) {
        cout << "weak_ptr is invalid" << endl;
    }

    return 0;
}


/* 
[wolfdan@cloud cpp]$ g++ A.cpp -o A.o -g --std=c++11 
[wolfdan@cloud cpp]$ ./A.o 
10
before reset: 2
after reset: 0
expired ? : 0
after reset: 0
0x13af028 10
not expired, sleep 1s...
not expired, sleep 1s...
not expired, sleep 1s...
^C
[wolfdan@cloud cpp]$ uname -a
Linux cloud 3.10.0-1127.el7.x86_64 #1 SMP Tue Mar 31 23:36:51 UTC 2020 x86_64 x86_64 x86_64 GNU/Linux
[wolfdan@cloud cpp]$ g++ --version
g++ (GCC) 4.8.5 20150623 (Red Hat 4.8.5-39)
Copyright (C) 2015 Free Software Foundation, Inc.
This is free software; see the source for copying conditions.  There is NO
warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.


[wolfdan@cloud cpp]$ g++ A.cpp -o A.o -g --std=c++11 
[wolfdan@cloud cpp]$ ./A.o 
10
before reset: 2
after reset: 0
after reset: sptr count 1
after sptr reset: sptr count 0
expired ? : 1
after reset: 0
weak_ptr is invalid
weak_ptr is invalid
 */
```

